#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <gsl/gsl_math.h>
#include <gsl/gsl_vector.h>
#include <gsl/gsl_matrix.h>
#include <gsl/gsl_linalg.h>
#include <gsl/gsl_eigen.h>
#include <gsl/gsl_blas.h>
#include <gsl/gsl_rng.h>
#include <gsl/gsl_randist.h>


#define MAX_LOC         100     //i={1,2,...,n}
#define MAX_LOC_OBS     100     //j={1,2,...,q}
#define MAX_COEFF       10      //k={1,2,...,p}
#define MAX_DIM         3       //maximum dimension of the spatial data
#define MAX_PC          10      //maximum number of principle components
#define MAX_MCREC       3000    //maximum number of MCMC result records

#define PCA_CONTRIBUTE_RATE 0.90    //set up PCA contribute rate

typedef struct Prior     prior;
typedef struct Parameter parameter;

struct Prior{
    int    InvWish1Dof;               //v_{Sigma}, Prior for Sigma
    gsl_matrix *InvWish1Scale;           //S_{Sigma}, Prior for Sigma
    int    InvWish2Dof;                //v_{Omega}, Prior for beta_{0, 1~q}
    gsl_matrix *InvWish2Scale;           //S_{Omega}, Prior for beta_{0, 1~q}
    double InvGammaShape;                //a: Prior for sigma_{0, 1~q}
    double InvGammaScale;                //b: Prior for sigma_{0, 1~q}
    double UnifLower;                    //c: Prior for rho
    double UnifUpper;                    //d: Prior for rho
};

struct Parameter{
    gsl_matrix *AugData;             //truncated Gaussian, z_{ij}
    gsl_matrix *AugDataVar;          //Inverse Wishart,    Sigma
    gsl_matrix *coeff;               //Gaussian            beta_{1~q}, p*1
    gsl_vector *coeff0;              //Gaussian            beta_{0},
    gsl_matrix *spatl;               //Gaussian            alpha_{1~q}, l*1
    gsl_vector *spatl0;              //Gaussian            alpha_{0}
    gsl_matrix **coeffVar;           //Inverse Wishart,    Omega_{1~q}
    gsl_matrix *coeffVar0;           //Inverse Wishart,    Omega_{0}
    gsl_vector *spatlVar;            //Inverse Gamma,      sigma_{1~q}
    double spatlVar0;                //Inverse Gamma,      sigma_{0}
    double radius;                   //rho for spatial correlation effect modeling
};

prior      *PriorPnt;
parameter  *ParameterPnt;

int  LOC       =-1;    //Total number of locations (i.e. spatial information),
int  LOC_OBS   =-1;    //Total number of observations at fixed lications, q
int  COEFF     =-1;    //Number of coefficients, p
int  DIM       =-1;    //Dimension of spatial data
int  MC_t      =-1;    //times of MCMC
int  MC_RecGap =-1;    //Number of iterations for recording
int  BurnIn    =-1;    //Numbers of burning in interations
long rndSeed   =-1;    //Seed of random number generator
int  PC_NUM    =-1;    //number of principle components, to be determined

int    Bi_Reaction[MAX_LOC][MAX_LOC_OBS]; //binary reaction in the spatial map (delta_{ij})
double SpatialData[MAX_LOC][MAX_DIM] ;     //spatial data
//double Covariate  [MAX_LOC][MAX_COEFF];   //record of covariates
//double SpatialEVec[MAX_LOC][MAX_PC];      //principle component eigenvectors for each location
//double SpatialEVal[MAX_PC];               //eigenvalues for each principle component
gsl_vector *eval;   // lamda : first PC_NUM eigen values of the the GP
gsl_matrix *evec;   // phi: Design matrix of GP
gsl_matrix *covr;   // w:   Design matrix of covariates


gsl_matrix *gsl_mat_diag(gsl_vector *X){
    //Creating Matrix = diag( vector )
    gsl_matrix * mat = gsl_matrix_alloc(X->size, X->size);
    gsl_vector_view diag = gsl_matrix_diagonal(mat);
    gsl_matrix_set_all(mat, 0.0); //or whatever number you like
    gsl_vector_memcpy(&diag.vector, X);
    return mat;
}
gsl_matrix *MatrixInv (gsl_matrix * M){
    // Returning the inverse matrix of M
    // M must be symmetric PD;
   gsl_matrix *buffer = gsl_matrix_alloc(M->size1, M->size2);
   gsl_matrix_memcpy(buffer, M);

   gsl_linalg_cholesky_decomp(buffer);
   gsl_linalg_cholesky_invert(buffer);

   return(buffer);
}
gsl_matrix *DiagMatrixInv (gsl_matrix *M){
    // inverse of a diagnal matrix
    int i;
    double temp;
    gsl_matrix *buffer = gsl_matrix_calloc(M->size1, M->size2);
    for(i=0; i<M->size1; i++){
        temp = gsl_matrix_get(M, i, i);
        temp = 1/temp;
        gsl_matrix_set(buffer, i, i, temp);
    }
    return(buffer);
}
double LinQuad (gsl_vector *b, gsl_matrix *A, gsl_vector *c){
    //calculating b'inv(A)c, where A is Symmetric PD.
    double *linquad;
    linquad =(double*) malloc(sizeof(double));
    gsl_vector *x;

    gsl_matrix *buffer = gsl_matrix_alloc(A->size1, A->size2); //Avoid the effect on matrix A
    gsl_matrix_memcpy(buffer, A);

    x=gsl_vector_alloc(b->size);
    gsl_linalg_cholesky_decomp(buffer);
    gsl_linalg_cholesky_solve(buffer, c, x);  // inv(A)*c
    gsl_blas_ddot(b, x, linquad);        //b'*inv(A)*c
    return (*linquad);
}
gsl_vector *VecDropID(gsl_vector *b, int id){
    //drop b[i == id-1], return b, i.e. drop the id th element of the vector
    int i; gsl_vector *p;
    p = gsl_vector_alloc(b->size-1);
    double *a = (double*)malloc((b->size-1)*sizeof(double));
    for(i=0; i<b->size; i++){
        if(i<  (id-1)) {a[i]  =gsl_vector_get(b, i);}
        if(i>  (id-1)) {a[i-1]=gsl_vector_get(b, i);}
    }
    for(i=0; i<b->size-1; i++)
        gsl_vector_set(p, i, a[i]);
    free(a); return(p); free(p);
}
gsl_matrix *MatDropID(gsl_matrix *A, int id){
    //drop A[i == id-1][j == id-1], return A
    //i.e. drop the id th line and the id th row of the matrix
    int i, j; gsl_matrix *p;
    p = gsl_matrix_alloc(A->size1-1, A->size2-1);
    double **a= (double**)malloc((A->size1-1)*sizeof(double*));
    for(i =0; i<A->size1; i++){
        if(i< id-1){
            a[i] =(double *)malloc((A->size2-1)*sizeof(double));
            for(j=0; j<A->size2; j++){
                if(j<id-1) a[i][j]   = gsl_matrix_get(A, i, j);
                if(j>id-1) a[i][j-1] = gsl_matrix_get(A, i, j);
            }
        }
        if(i> id-1){
            a[i-1]=(double *)malloc((A->size2-1)*sizeof(double));
            for(j=0; j<A->size2; j++){
                if(j<id-1) a[i-1][j]  = gsl_matrix_get(A, i, j);
                if(j>id-1) a[i-1][j-1]= gsl_matrix_get(A, i, j);
            }
        }
    }
    for(i=0; i<A->size1-1; i++)
        for(j=0; j<A->size2-1; j++)
            gsl_matrix_set(p, i, j, a[i][j]);
    free(a); return(p);
}
gsl_vector *DiagTimesVect(gsl_matrix *Lambda, gsl_vector *b){
    //Lambda: diagnal matrix
    //Calculate Lambda * b
    int i;
    double temp;
    gsl_vector *result = gsl_vector_alloc(b->size);
    for(i=0; i<b->size; i++){
        temp = gsl_matrix_get(Lambda, i, i)*gsl_vector_get(b, i);
        gsl_vector_set(result, i, temp);
    }
    return(result);
}

int rmvnorm(const gsl_rng *r, const int n, const gsl_vector *mean, const gsl_matrix *var, gsl_vector *result){
/* multivariate normal distribution random number generator */
/*
*	n	dimension of the random vetor
*	mean	vector of means of size n
*	var	variance matrix of dimension n x n
*	result	output variable with a sigle random vector normal distribution generation
*/
    int k;
    gsl_matrix *work = gsl_matrix_alloc(n,n);

    gsl_matrix_memcpy(work,var);
    gsl_linalg_cholesky_decomp(work);
    for(k=0; k<n; k++)
	    gsl_vector_set( result, k, gsl_ran_ugaussian(r) );

    gsl_blas_dtrmv(CblasLower, CblasNoTrans, CblasNonUnit, work, result);
    gsl_vector_add(result,mean);

    gsl_matrix_free(work);
    return(1);
}

int rwishart(const gsl_rng *r, const int n, const int dof, const gsl_matrix *scale, gsl_matrix *result){
/* Wishart distribution random number generator */
/*
*	n	 gives the dimension of the random matrix
*	dof	 degrees of freedom
*	scale	 scale matrix of dimension n x n
*	result	 output variable with a single random matrix Wishart distribution generation
*/
    int k,l;
    gsl_matrix *work = gsl_matrix_calloc(n,n);

    for(k=0; k<n; k++){
	    gsl_matrix_set( work, k, k, sqrt( gsl_ran_chisq( r, (dof-k) ) ) );
	    for(l=0; l<k; l++){
		    gsl_matrix_set( work, k, l, gsl_ran_ugaussian(r) );
	    }
    }
    gsl_matrix_memcpy(result,scale);
    gsl_linalg_cholesky_decomp(result);
    gsl_blas_dtrmm(CblasLeft,CblasLower,CblasNoTrans,CblasNonUnit,1.0,result,work);
    gsl_blas_dsyrk(CblasUpper,CblasNoTrans,1.0,work,0.0,result);
    return(1);
}

//Kernel decomposition
int KernelRepresent (int dimension, const double contribute_rate){
    int i; int j; int k;
    double kernel_ij;
    double dist;
    double sum;
    sum  = 0;
    dist = 0;
    gsl_matrix *kernel = gsl_matrix_alloc (LOC, LOC);
    for(i=0; i<LOC; i++)
        gsl_matrix_set(kernel, i, i, 1);
    for(i=0; i<LOC; i++){
        for(j=i+1; j<LOC; j++){
            //exp(-sqrt((x[i]-x[j])^2+(y[i]-y[j])^2)/rho)
            for(k=0; k<dimension; k++){
                dist = dist + (SpatialData[i][k]-SpatialData[j][k])*(SpatialData[i][k]-SpatialData[j][k]);
            }
            dist=sqrt(dist)/ParameterPnt->radius;
            kernel_ij = 1/exp(dist);
            dist=0;
            gsl_matrix_set(kernel, i, j, kernel_ij);
        }
    }
    gsl_vector *kernel_eval = gsl_vector_alloc(LOC);
    gsl_matrix *kernel_evec = gsl_matrix_alloc(LOC, LOC);

    gsl_eigen_symmv_workspace *w = gsl_eigen_symmv_alloc(LOC);
    gsl_eigen_symmv(kernel, kernel_eval, kernel_evec, w);
    gsl_eigen_symmv_sort(kernel_eval, kernel_evec, GSL_EIGEN_SORT_VAL_DESC);
    gsl_eigen_symmv_free(w);

    for(i=0; i<LOC; i++){
        sum = sum + gsl_vector_get(kernel_eval, i);
    }
    dist = 0; //dist here is used as the sum of first few eigen values
    for(i=0; i<LOC; i++){
        dist = dist + gsl_vector_get(kernel_eval, i);
        if(dist/sum > contribute_rate)
            break;
    }
    PC_NUM = i;
    eval = gsl_vector_alloc(PC_NUM);
    evec = gsl_matrix_alloc(LOC, PC_NUM);
    for(i=0; i<LOC; i++){
        for(j=0; j<PC_NUM; j++){
            gsl_matrix_set(evec, i, j, gsl_matrix_get(kernel_evec, i, j));
            gsl_vector_set(eval, j, gsl_vector_get(kernel_eval, j));
        }
    }
    gsl_vector_free(kernel_eval);
    gsl_matrix_free(kernel_evec);
    return(1);
}

int CalTrucNorm_mu_sigma(int locID, int obsID, double *TrucNorm_mu, double *TrucNorm_sigma){

    double mu;
    double sigma;
    //alpha: ParameterPnt->spatl
    //phi  : evec
    gsl_vector *alpha_times_phi = gsl_vector_calloc(LOC_OBS);
    gsl_vector *phi_locID = gsl_vector_alloc(PC_NUM);
    gsl_matrix_get_row(phi_locID, evec, locID);//phi_{i}

    //beta : ParameterPnt->coeff
    //w    : covr
    gsl_vector *beta_times_w    = gsl_vector_calloc(LOC_OBS);
    gsl_vector *w_locID = gsl_vector_alloc(COEFF);
    gsl_matrix_get_row(w_locID, covr, locID); //w_{i}

    gsl_vector *z_locID = gsl_vector_alloc(LOC_OBS);
    gsl_matrix_get_row(z_locID, ParameterPnt->AugData, locID);  // z_{i}

    gsl_blas_dgemv(CblasNoTrans, 1.0, ParameterPnt->spatl, phi_locID, 0.0, alpha_times_phi); //alpha*phi_{i}
    gsl_blas_dgemv(CblasNoTrans, 1.0, ParameterPnt->coeff, w_locID,   0.0, beta_times_w);    //beta*w_{i}  //gsl_ref P.126

    gsl_vector_sub(z_locID, alpha_times_phi);  //z_locID = z_{i}-alpha*phi_{i}1Ŕ    gsl_matrix_sub(z_locID, beta_times_w);     //z_locID = z_{i}-alpha*phi_{i}-beta*w_{i}
    gsl_vector *z_locID_drop_obsID = VecDropID(z_locID, obsID); //z_{i,-j}-alpha_{-j}*phi_{i}-beta_{-j}*w_{i}

    gsl_matrix *Sigma_Drop_obsID = MatDropID(ParameterPnt->AugDataVar, obsID); //Sigma_{-j, -j}
    gsl_vector *Sigma_obsID = gsl_vector_alloc(LOC_OBS);
    gsl_matrix_get_row(Sigma_obsID, ParameterPnt->AugDataVar, obsID);   //Sigma_{j}
    gsl_vector *Sigma_obsID_Drop_obsID = VecDropID(Sigma_obsID, obsID); //Sigma_{j,-j}

    mu = LinQuad(Sigma_obsID_Drop_obsID, Sigma_Drop_obsID, z_locID_drop_obsID);
    mu = mu + gsl_vector_get(alpha_times_phi, obsID); // -alpha_{j}*phi_{i}
    mu = mu + gsl_vector_get(beta_times_w,    obsID); // -w_{j}*beta_{i}

    sigma = LinQuad(Sigma_obsID_Drop_obsID, Sigma_Drop_obsID, Sigma_obsID_Drop_obsID);
    sigma = gsl_matrix_get(ParameterPnt->AugDataVar, obsID, obsID) - sigma;

    *TrucNorm_mu    = mu;
    *TrucNorm_sigma = sigma;

    return(1);
}

int UpdateAugData(const gsl_rng *r, int locID, int obsID){
    // n location ID, locID: i
    // q obs ID, obsID: j
    double *TrucNorm_mu   = (double*)malloc(sizeof(double));
    double *TrucNorm_sigma= (double*)malloc(sizeof(double));
    double LowerBound;
    double z_locID_obsID;

    CalTrucNorm_mu_sigma(locID, obsID, TrucNorm_mu, TrucNorm_sigma);

    /**Sampling from the Truncated Normal**/
    if(Bi_Reaction[locID][obsID] == 1){
        LowerBound = -(*TrucNorm_mu);
        z_locID_obsID = gsl_ran_gaussian_tail(r, LowerBound,  *TrucNorm_sigma); // >= -TrucNorm_mu
        z_locID_obsID = *TrucNorm_mu + z_locID_obsID ;  // >=0
    }
ř    else if(Bi_Reaction[locID[obsID] == 0){
        LowerBound = *TrucNorm_mu;
        z_locID_obsID = gsl_ran_gaussian_tail(r, LowerBound,  *TrucNorm_sigma); // <= TrucNorm_mu
        z_locID_obsID = *TrucNorm_mu - z_locID_obsID ;  // <=0
    }
    else {
        printf("Wrong Value of Spatial Binary Response: Must be 0 or 1");
        return(-1); exit(1);
    }

    gsl_matrix_set(ParameterPnt->AugData, locID, obsID, z_locID_obsID);
    return(1);
}

int UpdateCoeff(const gsl_rng *r, int obsID){
    int i; //loop from 1 to n
    double temp;     //This temp value record the z_{ij} - mu_{i, -j} + beta_{j}'*w_{i}
    double sigma_inv; //This value is 1/Sigma_{-j}
    gsl_vector *sum_mu;    //Summation part of the mean
    gsl_matrix *sum_sigma; //Summation part of the variance

    gsl_vector *alpha_times_phi = gsl_vector_calloc(LOC_OBS);
    gsl_vector *phi_locID       = gsl_vector_calloc(PC_NUM);
    gsl_vector *beta_times_w    = gsl_vector_calloc(LOC_OBS);
    gsl_vector *w_locID         = gsl_vector_calloc(COEFF);
    gsl_vector *z_locID         = gsl_vector_calloc(LOC_OBS);

    gsl_vector *Sigma_obsID     = gsl_vector_calloc(LOC_OBS);
    gsl_matrix *Sigma_Drop_obsID = MatDropID(ParameterPnt->AugDataVar, obsID); //Sigma_{-j, -j}
    gsl_matrix_get_row(Sigma_obsID, ParameterPnt->AugDataVar, obsID);          //Sigma_{j}
    gsl_vector *Sigma_obsID_Drop_obsID = VecDropID(Sigma_obsID, obsID);        //Sigma_{j,-j}

    sigma_inv = LinQuad(Sigma_obsID_Drop_obsID, Sigma_Drop_obsID, Sigma_obsID_Drop_obsID);
    sigma_inv = gsl_matrix_get(ParameterPnt->AugDataVar, obsID, obsID) - sigma_inv;
    sigma_inv = 1/sigma_inv;   // 1/Sigma_{-j}

    gsl_matrix *A = gsl_matrix_alloc(COEFF, COEFF);
    gsl_matrix_memcpy(A, ParameterPnt->coeffVar[obsID]);  //A = Omega_{j}
    gsl_linalg_cholesky_decomp(A);
    gsl_linalg_cholesky_solve(A, ParameterPnt->coeff0, sum_mu);  // sum_mu = inv(Omega_{j}*beta_{0}
    gsl_matrix_free(A);

    sum_sigma = MatrixInv(ParameterPnt->coeffVar[obsID]); //sum_sigam = inv(Omega_{j})

    for(i=0; i<LOC; i++){

        gsl_matrix_get_row(phi_locID, evec, i);  //phi_{i}
        gsl_matrix_get_row(w_locID,   covr, i);  //w_{i}
        gsl_matrix_get_row(z_locID, ParameterPnt->AugData, i);  // z_{i}

        gsl_blas_dgemv(CblasNoTrans, 1.0, ParameterPnt->spatl, phi_locID, 0.0, alpha_times_phi); //alpha*phi_{i}
        gsl_blas_dgemv(CblasNoTrans, 1.0, ParameterPnt->coeff, w_locID,   0.0, beta_times_w);    //beta*w_{i}  //see gsl_ref P.126

        gsl_matrix_sub(z_locID, alpha_times_phi);  //z_locID = z_{i}-alpha*phi_{i}
        gsl_matrix_sub(z_locID, beta_times_w);     //z_locID = z_{i}-alpha*phi_{i}-beta*w_{i}
        gsl_vector *z_locID_drop_obsID = VecDropID(z_locID, obsID); //z_{i,-j}-alpha_{-j}*phi_{i}-beta_{-j}*w_{i}

        temp = LinQuad(Sigma_obsID_Drop_obsID, Sigma_Drop_obsID, z_locID_drop_obsID);
        temp = temp + gsl_vector_get(alpha_times_phi, obsID); //+alpha_{j}*phi_{i}
        //mu = mu + gsl_vector_get(beta_times_w,    obsID); // w_{j}*beta_{i}
        temp = gsl_matrix_get(ParameterPnt->AugData, i, obsID) - temp;
        gsl_vector_scale(w_locID, temp*sigma_inv);  // w_locID = (z_{ij}-mu_{i,-j}+beta_{j}*w_{i})*w_{i}*inv(Sigma_{-j})
        gsl_vector_add(sum_mu, w_locID);

        gsl_blas_dger(sigam_inv, w_locID, w_locID, sum_sigma); // sum_sigma = sigma_inv * (w_locID*w_locID') + sum_sigma //see gsl_ref P.127

        gsl_vector_set_zero(alpha_times_phi);
        gsl_vector_set_zero(beta_times_w);
        gsl_vector_set_zero(phi_locID);
        gsl_vector_set_zero(w_locID);
        gsl_vector_set_zero(z_locID);
        temp = 0;
    }

    gsl_matrix *MVNorm_sigma_beta = MatrixInv(sum_sigma);
    gsl_vector *MVNorm_mu_beta = gsl_vector_calloc(COEFF);
    gsl_blas_dgemv(CblasNoTrans, 1.0, MVNorm_sigma_beta, sum_mu, 0.0, MVNorm_mu_beta);

    gsl_vector *beta_obsID = gsl_vector_calloc(COEFF);
    rmvnorm(r, COEFF, MVNorm_mu_beta, MVNorm_sigma_beta, beta_obsID);

    gsl_matrix_set_row(ParameterPnt->coeff, obsID, beta_obsID); return(1);
}

int UpdateSpatl(const gsl_rng *r, int obsID){
    int i; //loop from 1 to n
    double temp;     //This temp value record the z_{ij} - mu_{i, -j} + beta_{j}'*w_{i}
    double sigma_inv; //This value is 1/Sigma_{-j}
    gsl_vector *sum_mu;    //Summation part of the mean
    gsl_matrix *sum_sigma; //Summation part of the variance

    gsl_vector *alpha_times_phi = gsl_vector_calloc(LOC_OBS);
    gsl_vector *phi_locID       = gsl_vector_calloc(PC_NUM);
    gsl_vector *beta_times_w    = gsl_vector_calloc(LOC_OBS);
    gsl_vector *w_locID         = gsl_vector_calloc(COEFF);
    gsl_vector *z_locID         = gsl_vector_calloc(LOC_OBS);

    gsl_vector *Sigma_obsID     = gsl_vector_calloc(LOC_OBS);
    gsl_matrix *Sigma_Drop_obsID = MatDropID(ParameterPnt->AugDataVar, obsID); //Sigma_{-j, -j}
    gsl_matrix_get_row(Sigma_obsID, ParameterPnt->AugDataVar, obsID);          //Sigma_{j}
    gsl_vector *Sigma_obsID_Drop_obsID = VecDropID(Sigma_obsID, obsID);        //Sigma_{j,-j}

    sigma_inv = LinQuad(Sigma_obsID_Drop_obsID, Sigma_Drop_obsID, Sigma_obsID_Drop_obsID);
    sigma_inv = gsl_matrix_get(ParameterPnt->AugDataVar, obsID, obsID) - sigma_inv;
    sigma_inv = 1/sigma_inv;   // 1/Sigma_{-j}

    gsl_matrix *lmd = gsl_mat_diag(eval);   //lmd = LMBDA = diag{ eigen values }
    double sigma_obsID = gsl_vector_get(ParameterPnt->spatlVar, obsID); //sigma_obsID = sigma_{j}
    gsl_matrix_scale(lmd, sigam_obsID);     //lmd = sigma_{j} * diag{ eigen values }
    sum_sigma = DiagMatrixInv(lmd);         //sum_sigma = 1/sigma_{j} * inv(LAMBDA)
    sum_mu    = DiagTimesVect(lmd, ParameterPnt->spatl0);  // 1/sigma_{j} * inv(LAMBDA) * alpha_{0}

    for(i=0; i<LOC; i++){

        gsl_matrix_get_row(phi_locID, evec, i);  //phi_{i}
        gsl_matrix_get_row(w_locID,   covr, i);  //w_{i}
        gsl_matrix_get_row(z_locID, ParameterPnt->AugData, i);  // z_{i}

        gsl_blas_dgemv(CblasNoTrans, 1.0, ParameterPnt->spatl, phi_locID, 0.0, alpha_times_phi); //alpha*phi_{i}
        gsl_blas_dgemv(CblasNoTrans, 1.0, ParameterPnt->coeff, w_locID,   0.0, beta_times_w);    //beta*w_{i}  //see gsl_ref P.126

        gsl_matrix_sub(z_locID, alpha_times_phi);  //z_locID = z_{i}-alpha*phi_{i}
        gsl_matrix_sub(z_locID, beta_times_w);     //z_locID = z_{i}-alpha*phi_{i}-beta*w_{i}
        gsl_vector *z_locID_drop_obsID = VecDropID(z_locID, obsID); //z_{i,-j}-alpha_{-j}*phi_{i}-beta_{-j}*w_{i}

        temp = LinQuad(Sigma_obsID_Drop_obsID, Sigma_Drop_obsID, z_locID_drop_obsID);
        //temp = temp + gsl_vector_get(alpha_times_phi, obsID); // alpha_{j}*phi_{i}
        temp = temp + gsl_vector_get(beta_times_w,    obsID); // +w_{j}*beta_{i}
        temp = gsl_matrix_get(ParameterPnt->AugData, i, obsID) - temp;
        gsl_vector_scale(phi_locID, temp*sigma_inv);  // phi_locID = (z_{ij}-mu_{i,-j}+alpha_{j}*phi_{i})*phi_{i}*inv(Sigma_{-j})
        gsl_vector_add(sum_mu, phi_locID);

        gsl_blas_dger(sigam_inv, phi_locID, phi_locID, sum_sigma); // sum_sigma = sigma_inv * (phi_locID*phi_locID') + sum_sigma //see gsl_ref P.127

        gsl_vector_set_zero(alpha_times_phi);
        gsl_vector_set_zero(beta_times_w);
        gsl_vector_set_zero(phi_locID);
        gsl_vector_set_zero(w_locID);
        gsl_vector_set_zero(z_locID);
        temp = 0;
    }

    gsl_matrix *MVNorm_sigma_alpha = MatrixInv(sum_sigma);
    gsl_vector *MVNorm_mu_alpha = gsl_vector_calloc(PC_NUM);
    gsl_blas_dgemv(CblasNoTrans, 1.0, MVNorm_sigma_alpha, sum_mu, 0.0, MVNorm_mu_alpha);

    gsl_vector *alpha_obsID = gsl_vector_calloc(PC_NUM);
    rmvnorm(r, PC_NUM, MVNorm_mu_alpha, MVNorm_sigma_alpha, alpha_obsID);

    gsl_matrix_set_row(ParameterPnt->spatl, obsID, alpha_obsID); return(1);
}

int UpdateCoeff0(const gsl_rng *r){
    gsl_vector *sum_mu    = gsl_vector_calloc(COEFF);
    gsl_matrix *sum_sigma = MatrixInv(ParameterPnt->coeffVar0);  //Omega_{0}
    int j;
    gsl_matrix *Omega_inv        = gsl_matrix_alloc(COEFF, COEFF);
    gsl_vector *Solve_Omega_beta = gsl_vector_alloc(COEFF);
    gsl_vector *beta_j           = gsl_vector_alloc(COEFF);

    for(j = 0; j<LOC_OBS; j++){
        Omega_inv = MatrixInv(ParameterPnt->coeffVar[j]);
        gsl_matrix_add(sum_sigma, Omega_inv);

        gsl_matrix_get_row(beta_j, ParameterPnt->coeff, j);      //get beta_{j}
        gsl_blas_dgemv(CblasNoTrans, 1.0, Omega_inv, beta_j, 0.0, Solve_Omega_beta);  // Solve_Omega_beta = inv(Omega)*beta_{j}
        gsl_vector_add(sum_mu, Solve_Omega_beta);

        gsl_vector_set_zero(Solve_Omega_beta);
        gsl_vector_set_zero(beta_j);
    }

    gsl_matrix *MVNorm_beta0_sigma = MatrixInv(sum_sigma);
    gsl_vector *MVNorm_beta0_mu    = gsl_vector_calloc(COEFF);
    gsl_blas_dgemv(CblasNoTrans, 1.0, MVNorm_beta0_sigma, sum_mu, 0.0, MVNorm_beta0_mu);

    gsl_vector *beta0 = gsl_vector_alloc(COEFF);
    rmvnorm(r, COEFF, MVNorm_beta0_mu, MVNorm_beta0_sigma, beta0);

    ParameterPnt->coeff0 = beta0;
    return(1);
}

int UpdateSpatl0(const gsl_rng *r){
    gsl_vector *MVNorm_alpha0_mu    = gsl_vector_calloc(PC_NUM);
    int j;
    double sum_sigma_inv = 1/ParameterPnt->spatlVar0;        //1/sigma_{0}
    double sigma_inv;
    gsl_vector *alpha_j = gsl_vector_alloc(PC_NUM);

    for(j=0; j<LOC_OBS; j++){
        sigma_inv     = 1/gsl_vector_get(ParameterPnt->spatlVar, j);
        sum_sigma_inv = sum_sigma_inv + sigma_inv;

        gsl_matrix_get_row(alpha_j, ParameterPnt->spatl, j);
        gsl_vector_scale(alpha_j, sigma_inv);        // alpha_{j} := 1/sigma_{j} * alpha_{j}
        gsl_vector_add(MVNorm_alpha0_mu, alpha_j);

        gsl_vector_set_zero(alpha_j);
    }

    sum_sigma_inv = 1/sum_sigma_inv * 1/(1+LOC_OBS);
    gsl_matrix *MVNorm_alpha0_sigma= gsl_mat_diag(eval);    //diag{ eigen values }
    gsl_matrix_scale(MVNorm_alpha0_sigma, sum_sigma_inv);
    gsl_vector_scale(MVNorm_alpha0_mu,    sum_sigma_inv);

    gsl_vector *alpha0 = gsl_vector_alloc(PC_NUM);
    rmvnorm(r, PC_NUM, MVNorm_alpha0_mu, MVNorm_alpha0_sigma, alpha0);

    ParameterPnt->spatl0 = alpha0;
    return(1);
}

int UpdateCoeffVar(const gsl_rng *r, int obsID){
    gsl_vector *beta_obsID = gsl_vector_alloc(COEFF);
    gsl_matrix_get_row(beta_obsID, ParameterPnt->coeff, obsID) // beta_{j}
    gsl_vector_sub(beta_obsID, ParameterPnt->coeff0);   // beta_{j} - beta_{0}

    gsl_matrix *InvWishart_Omega = gsl_matrix_alloc(COEFF, COEFF);
    gsl_matrix_memcpy(InvWishart_Omega, PriorPnt->InvWish2Scale);

    gsl_blas_dger(1.0, phi_locID, phi_locID, InvWishart_Omega);

    int dof = PriorPnt->InvWish2Dof + 1;

    gsl_matrix *result = gsl_matrix_alloc(COEFF, COEFF);
    rwishart(r, COEFF, dof, InvWishart_Omega, result);  //generate Wishart

    ParameterPnt->coeffVar[obsID] = MatrixInv(result);  //inverse Wishart
    return(1);
}

int UpdateSpatlVar(const gsl_rng *r, int obsID){
    gsl_vector *alpha_obsID = gsl_vector_alloc(PC_NUM);
    gsl_matrix_get_row(alpha_obsID, ParameterPnt->spatl, obsID); //alpha_{j}
    gsl_vector_sub(alpha_obsID, ParameterPnt->spatl0);

    double InvGamma_a = PriorPnt->InvGammaShape + 1;
    double InvGamma_b = PriorPnt->InvGammaScale;
    gsl_matrix *lmd = gsl_mat_diag(eval);

    double added = LinQuad(alpha_obsID, lmd, alpha_obsID);
    InvGamma_b = InvGamma_b + 1/2*added;

    double result = gsl_ran_gamma_knuth(r, InvGamma_a, InvGamma_b);
    result = 1/result;

    gsl_vector_set(ParameterPnt->spatlVar, result, obsID);
    return(1);
}

int UpdateCoeffVar0(const gsl_rng *r){
    gsl_vector *beta0 = gsl_vector_alloc(COEFF);
    gsl_vector_memcpy(beta0, ParameterPnt->coeff0);      // beta_{0}

    gsl_matrix *InvWishart_Omega = gsl_matrix_alloc(COEFF, COEFF);
    gsl_matrix_memcpy(InvWishart_Omega, PriorPnt->InvWish2Scale); // S_{Omega} added in

    gsl_blas_dger(1.0, phi_locID, phi_locID, InvWishart_Omega);

    int dof = PriorPnt->InvWish2Dof + 1;

    gsl_matrix *result = gsl_matrix_alloc(COEFF, COEFF);
    rwishart(r, COEFF, dof, InvWishart_Omega, result);  //generate Wishart

    ParameterPnt->coeffVar0 = MatrixInv(result);  //inverse Wishart
    return(1);

}

int UpdateSpatlVar0(const gsl_rng *r){

    double InvGamma_a = PriorPnt->InvGammaShape + 1;
    double InvGamma_b = PriorPnt->InvGammaScale;
    gsl_matrix *lmd = gsl_mat_diag(eval);

    double added = LinQuad(ParameterPnt->spatl0, lmd, ParameterPnt->spatl0);
    InvGamma_b = InvGamma_b + 1/2*added;

    double result = gsl_ran_gamma_knuth(r, InvGamma_a, InvGamma_b);
    result = 1/result;

    gsl_vector_set(ParameterPnt->spatlVar, result, obsID);
    return(1);
}

int UpdateAugDataVar(const gsl_rng *r){
    int i;

    gsl_matrix *Scale   = gsl_matrix_alloc(LOC_OBS, LOC_OBS);
    gsl_matrix_memcpy(Scale, PriorPnt->InvWish1Scale);    // Scale initially S_{Omega}

    gsl_vector *alpha_times_phi = gsl_vector_calloc(LOC_OBS);
    gsl_vector *phi_locID       = gsl_vector_calloc(PC_NUM);
    gsl_vector *beta_times_w    = gsl_vector_calloc(LOC_OBS);
    gsl_vector *w_locID         = gsl_vector_calloc(COEFF);
    gsl_vector *z_locID         = gsl_vector_calloc(LOC_OBS);

    for(i=0; i<LOC; i++){
        gsl_matrix_get_row(z_locID, ParameterPnt->AugData, i); //z_locID = z_{i}

        gsl_matrix_get_row(phi_locID, evec, i);  //phi_{i}
        gsl_matrix_get_row(w_locID,   covr, i);  //w_{i}
        gsl_matrix_get_row(z_locID, ParameterPnt->AugData, i);  // z_{i}

        gsl_blas_dgemv(CblasNoTrans, 1.0, ParameterPnt->spatl, phi_locID, 0.0, alpha_times_phi); //alpha*phi_{i}
        gsl_blas_dgemv(CblasNoTrans, 1.0, ParameterPnt->coeff, w_locID,   0.0, beta_times_w);    //beta*w_{i}  //see gsl_ref P.126

        gsl_matrix_sub(z_locID, alpha_times_phi);
        gsl_matrix_sub(z_locID, beta_times_w);

        gsl_blas_dger(0.5, z_locID, z_locID, Scale);
    }

    double dof = PriorPnt->InvWish1Dof + LOC;

    gsl_matrix *result = gsl_matrix_alloc(LOC_OBS, LOC_OBS);
    rwishart(r, LOC_OBS, dof, Scale, result);

    ParameterPnt->AugDataVar = result;
    return(1);
}

/**
int UpdateRadius(const gsl_rng *r){
}
**/

int File_ReadIn (int argc, char **argv){
    int i=0; int j=0;
    if (argc != 6){
        printf("##########################################################\n");
        printf("Wrong input formating! See instructions:\n");
        printf("general_par  prior  bin_response  sptl_coord  covariate  parameter_initial_guess\n");
        printf("##########################################################\n");
        return(-1);exit(1);
    }
    FILE *fpPar, *fpPrior, *fpBiRsp, *fpSpt, *fpCov, *fpPar0;

    if((fpPar=fopen(argv[1], "r+"))==NULL){
        printf("Open initial parameter file error!");
        return(-1); exit(1);
    }
    else{
        fscanf(fpPar, "%d", &LOC);
        fscanf(fpPar, "%d", &LOC_OBS);
        fscanf(fpPar, "%d", &COEFF);
        fscanf(fpPar, "%d", &DIM);
        fscanf(fpPar, "%d", &MC_t);
        fscanf(fpPar, "%d", &MC_RecGap);
        fscanf(fpPar, "%d", &BurnIn);
        fscanf(fpPar, "%ld",&rndSeed);
    }
    fclose(fpPar);
    /***Readin Prior Parameters***/
    if((fpPrior=fopen(argv[2], "r+"))==NULL){
        printf("Open prior parameter file error!");
        return(-1); exit(1);
    }
    else{
        PriorPnt = (prior*)malloc(sizeof(prior));
        fscanf(fpPrior, "%d", &PriorPnt->InvWish1Dof);
        if(LOC_OBS < 0){
            printf("False input of q: observations at one spatial point!");
            return(-1); exit(1);
        }
        else{
            PriorPnt->InvWish1Scale = gsl_matrix_alloc(LOC_OBS, LOC_OBS);
            gsl_matrix_fscanf(fpPrior, PriorPnt->InvWish1Scale);
        }
        fscanf(fpPrior, "%d", &PriorPnt->InvWish2Dof);
        if( COEFF < 0){
            printf("False input covariate numbers!");
            return(-1); exit(1);
        }
        else{
            PriorPnt->InvWish2Scale = gsl_matrix_alloc(COEFF, COEFF);
            gsl_matrix_fscanf(fpPrior, PriorPnt->InvWish2Scale);
        }
        fscanf(fpPrior, "%lf", &PriorPnt->InvGammaShape);
        fscanf(fpPrior, "%lf", &PriorPnt->InvGammaScale);
        fscanf(fpPrior, "%lf", &PriorPnt->UnifLower);
        fscanf(fpPrior, "%lf", &PriorPnt->UnifUpper);
    }
    fclose(fpPrior);

    /***Readin Binary Response file***/
    if((fpBiRsp=fopen(argv[3], "r+"))==NULL){
        printf("Open binary respond file error!");
        return(-1); exit(1);
    }
    else{

        if(LOC < 0){
            printf("False input of n: numbers of total observed places!");
            return(-1); exit(1);
        }
        else{
            for(i=0; i<LOC; i++)
                for(j=0; j<LOC_OBS; j++)
                    fscanf(fpBiRsp, "%d", &Bi_Reaction[i][j]);
        }
    }
    fclose(fpPrior);

    /***Readin Spatial Coordinate ***/
    if((fpSpt=fopen(argv[4], "r+"))==NULL){
        printf("Open spatial coordinates file error!");
        return(-1); exit(1);
    }
    else{
        if(DIM < 0){
            printf("False input of dimension: dimension of spatial records!");
            return(-1); exit(1);
        }
        else{
            for(i=0; i<LOC; i++)
                for(j=0; j<DIM; j++)
                    fscanf(fpSpt, "%lf", &SpatialData[i][j]);
        }
    }
    fclose(fpSpt);

    /***Readin Covariate Design Matrix ****/
    if((fpCov=fopen(argv[5], "r+"))==NULL){
        printf("Open covariate records file error!");
        return(-1); exit(1);
    }
    else{
        covr = gsl_matrix_alloc(LOC, COEFF);
        gsl_matrix_fscanf(fpCov, covr);
    }
    fclose(fpCov);

    /****Readin Initial Guess of the Parameters ***/
    if((fpPar0=fopen(argv[6], "r+"))==NULL){
        printf("Open parameter initial guess file error!");
        return(-1); exit(1);
    }
    else{
        if(PC_NUM < 0){
            printf("Eigen Decomposition Error: # of PC did not returned.");
            return(-1); exit(1);
        }
        else{
            //allocate memory space for the parameters
            ParameterPnt = (parameter*)malloc(sizeof(parameter));

            ParameterPnt->AugData    = gsl_matrix_alloc(LOC, LOC_OBS);
            ParameterPnt->AugDataVar = gsl_matrix_alloc(LOC_OBS, LOC_OBS);
            ParameterPnt->coeff      = gsl_matrix_alloc(LOC_OBS, COEFF);
            ParameterPnt->coeff0     = gsl_vector_alloc(COEFF);
            ParameterPnt->spatl      = gsl_matrix_alloc(LOC_OBS, PC_NUM);
            ParameterPnt->spatl0     = gsl_vector_alloc(PC_NUM);
            ParameterPnt->coeffVar   =(gsl_matrix **)malloc(LOC_OBS*sizeof(gsl_matrix *));
            ParameterPnt->coeffVar0  = gsl_matrix_alloc(COEFF, COEFF);
            ParameterPnt->spatlVar   = gsl_vector_alloc(LOC_OBS);

            gsl_matrix_fscanf(fpPar0, ParameterPnt->AugData);
            gsl_matrix_fscanf(fpPar0, ParameterPnt->AugDataVar);
            gsl_matrix_fscanf(fpPar0, ParameterPnt->coeff);
            gsl_vector_fscanf(fpPar0, ParameterPnt->coeff0);
            gsl_matrix_fscanf(fpPar0, ParameterPnt->spatl);
            gsl_vector_fscanf(fpPar0, ParameterPnt->spatl0);

            for(i=0; i<LOC_OBS, i++){
                ParameterPnt->coeffVar = gsl_matrix_alloc(COEFF, COEFF);
                gsl_matrix_fscanf(fpPar0, ParameterPnt->coeffVar[i]);
            }
            gsl_matrix_fscanf(fpPar0, ParameterPnt->coeffVar0);
            gsl_vector_fscanf(fpPar0, ParameterPnt->spatlVar);
            fscanf(fpPar0, "%lf", &ParameterPnt->spatlVar0);
            fscanf(fpPar0, "%lf", &ParameterPnt->radius);
        }
    }
    fclose(fpPar0);
    return(1);
}


int main(int argc, char ** argv){
    int mc_count;

    File_ReadIn(argc, argv);
    int Record_Number = (int)(MC_T/MC_RecGap);

    //allocate space for storing parameter
    parameter *parameter_record;
    parameter_record = (parameter *)malloc(Record_Number*sizeof(parameter));

    //setting up random number generator
    const gsl_rng_type * T;
    gsl_rng * r;
    gsl_rng_env_setup();
    T = gsl_rng_default;
    r = gsl_rng_alloc (T);

    KernelRepresent(DIM, PCA_CONTRIBUTE_RATE);
    int i; int j; int record_count;
    record_count = 0;
    for(mc_count=0; mc_count < MC_t; mc_count++){

        for(j= 0; j<LOC_OBS; j++){
            UpdateCoeff(r, j);
            UpdateSpatl(r, j);
            UpdateCoeffVar(r, j);
            UpdateSpatlVar(r, j);
            for(i= 0; i<LOC; i++){
                UpdateAugData(r, i, j);
            }
        }
        UpdateCoeff0(r);
        UpdateSpatl0(r);
        UpdateCoeffVar0(r);
        UpdateSpatlVar0(r);
        UpdateAugDataVar(r);

        if(mc_count%%MC_RecGap == 1){
            record_count ++;

            //Record MCMC chain after thinning
            gsl_matrix_memcpy(parameter_record[record_count].AugData,    ParameterPnt->AugData);
            gsl_matrix_memcpy(parameter_record[record_count].AugDataVar, ParameterPnt->AugDataVar);

            gsl_matrix_memcpy(parameter_record[record_count].coeff,  ParameterPnt->coeff);
            gsl_matrix_memcpy(parameter_record[record_count].spatl,  ParameterPnt->spatl);
            gsl_vector_memcpy(parameter_record[record_count].coeff0, ParameterPnt->coeff0);
            gsl_vector_memcpy(parameter_record[record_count].spatl0, ParameterPnt->spatl0);

            parameter_record[record_count].coeffVar = (gsl_matrix **)malloc(LOC_OBS*sizeof(gsl_matrix*));
            for(i=0; i<LOC_OBS; i++)
                gsl_matrix_memcpy(parameter_record[record_count].coeffVar[i], ParameterPnt->coeffVar[i]);
            gsl_matrix_memcpy(parameter_record[record_count].coeffVar0, ParameterPnt->coeffVar0);

            gsl_vector_memcpy(parameter_record[record_count].spatlVar, ParameterPnt->spatlVar);
            parameter_record[record_count].spatlVar0 = ParameterPnt->spatlVar0;

            parameter_record[record_count].radius = ParameterPnt->radius;
        }
    }
    Par_Record_File_WriteOut(parameter_record, Record_Number);
    gsl_rng_free (r);
    return(1);
}
